(in-package :cl-blog)

(defparameter *blog-pathname-root*
  (make-pathname :directory
                 (pathname-directory
                  (asdf:component-pathname
                   (asdf:find-component
		    nil
		    #+cl-modules
		    #.(cl-modules:current-module-name :downcase t)
		    #-cl-modules "cl-blog")))))

(defparameter *blog-configuration*
  (parse-configuration-file
   (merge-pathnames "blog.conf" *blog-pathname-root*)))

(defparameter *user-configuration*
  (parse-configuration-file (merge-pathnames "users.conf" *blog-pathname-root*)))

(defparameter *blog-template*
  (load-template (merge-pathnames (configuration-value *blog-configuration* :template :blog)
                                  *blog-pathname-root*)))

(defparameter *blog-short-name*
  (configuration-value *blog-configuration* :name :blog))

(defparameter *users*
  (mapcar #'car (cdr (configuration-value *user-configuration* :users))))

(defparameter *owner-email*
  (configuration-value *blog-configuration* :owner-email :blog))

(defparameter *categories*
  (configuration-value *blog-configuration* :categories :blog))

(defparameter *blog-url-root*
  (parse-urlstring (configuration-value *blog-configuration*
                                        :url-root :blog)))

(defparameter *internal-http-port*
  (configuration-value *blog-configuration* :internal-http-port :blog))

(defvar *passwords* nil)

(defparameter *entry-path*
  (merge-pathnames (make-pathname :directory '(:relative "entries"))
                   *blog-pathname-root*))

(defparameter *entry-index*
  (merge-pathnames "index" *entry-path*))

(defvar *listener*
  (let ((fwd-url (araneida:copy-url *blog-url-root*)))
    (setf (araneida:url-port fwd-url) *internal-http-port*)
    (make-instance #+sbcl
                   'araneida:serve-event-reverse-proxy-listener
                   #-sbcl
                   'araneida:threaded-reverse-proxy-listener
                   :translations
                   `((,(araneida:urlstring *blog-url-root*)
                      ,(araneida:urlstring fwd-url)))
                   :address #(127 0 0 1)
                   :port (araneida:url-port fwd-url))))

(defparameter *new-entry-url* (merge-url *blog-url-root* "new"))

(defparameter *display-entry-url* (merge-url *blog-url-root* "display/"))

(defparameter *archives-url* (merge-url *blog-url-root* "archives"))

(defparameter *rss-url* (merge-url *blog-url-root* "archives.rss"))

(defparameter *edit-entry-url* (merge-url *blog-url-root* "edit/"))

(defparameter *delete-entry-url* (merge-url *blog-url-root* "delete/"))

(defparameter *email-redirect-url* (merge-url *blog-url-root* "contact"))

(defparameter *trackback-url* (merge-url *blog-url-root* "trackback/"))
